@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from copy import deepcopy
-from typing import Any, Dict, List, NoReturn, Optional, Tuple, Type, Union, get_type_hints
+from typing import Any, Dict, List, NoReturn, Optional, Sequence, Type, Union, get_type_hints
 
 from pydantic import BaseConfig, BaseModel
 from pydantic.main import create_model
@@ -107,6 +107,9 @@ class _TableModelConfig(BaseConfig):
     arbitrary_types_allowed = True
 
 
+FetchType = Sequence[Sequence[_Relationship]]
+
+
 class Table:
 
     __tablename__: str
@@ -213,40 +216,72 @@ class Table:
     # Queries
     #
     @classmethod
-    async def insert_one(cls: Union[Type[Model], Table], /, item: Model) -> Model:
+    async def insert_one(
+        cls: Union[Type[Model], Table],
+        /,
+        item: Model,
+        *,
+        prefetch: FetchType = None,
+    ) -> Model:
 
         query: QueryBuilder = (
             Query.into(cls.__tablename__)
             .columns(cls._fields(exclude_autogen=True))
             .insert(*cls._db_values(item, exclude_autogen=True))
         )
-        return await Porm.fetch_one(with_returning(query), cls)
+
+        inserted: Optional[Model] = await Porm.fetch_one(with_returning(query), cls)
+
+        if prefetch and inserted:
+            [inserted] = await cls.fetch_related([inserted], prefetch)
+
+        return inserted
 
     @classmethod
-    async def insert_many(cls: Union[Type[Model], Table], /, items: List[Model]) -> List[Model]:
+    async def insert_many(
+        cls: Union[Type[Model], Table],
+        /,
+        items: List[Model],
+        *,
+        prefetch: FetchType = None,
+    ) -> List[Model]:
         query: QueryBuilder = Query.into(cls.__tablename__).columns(*cls._fields(exclude_autogen=True))
 
         for item in items:
             query = query.insert(*cls._db_values(item, exclude_autogen=True))
 
-        return await Porm.fetch_many(with_returning(query), cls)
+        inserted = await Porm.fetch_many(with_returning(query), cls)
+
+        if prefetch and inserted:
+            inserted = await cls.fetch_related(inserted, prefetch)
+
+        return inserted
 
     @classmethod
     async def fetch_first(
-        cls: Union[Type[Model], Table], /, criterion: Criterion, *, prefetch: Tuple[Tuple[_Relationship]] = None
+        cls: Union[Type[Model], Table],
+        /,
+        criterion: Criterion,
+        *,
+        prefetch: FetchType = None,
     ) -> Optional[Model]:
         query: QueryBuilder = cls.select().where(criterion)
         query = query.limit(1)
+
         result = await Porm.fetch_one(query.get_sql(), cls)
 
-        if prefetch:
-            await cls.fetch_related([result], prefetch)
+        if result and prefetch:
+            [result] = await cls.fetch_related([result], prefetch)
 
         return result
 
     @classmethod
     async def fetch_one(
-        cls: Union[Type[Model], Table], /, criterion: Criterion, *, prefetch: Tuple[Tuple[_Relationship]] = None
+        cls: Union[Type[Model], Table],
+        /,
+        criterion: Criterion,
+        *,
+        prefetch: FetchType = None,
     ) -> Model:
         query: QueryBuilder = cls.select().where(criterion)
         query = query.limit(2)
@@ -260,7 +295,7 @@ class Table:
         result = results[0]
 
         if prefetch:
-            await cls.fetch_related([result], prefetch)
+            [result] = await cls.fetch_related([result], prefetch)
 
         return result
 
@@ -270,7 +305,7 @@ class Table:
         /,
         criterion: Criterion = None,
         *,
-        prefetch: Tuple[Tuple[_Relationship]] = None,
+        prefetch: FetchType = None,
     ) -> List[Model]:
 
         query: QueryBuilder = cls.select()
@@ -281,12 +316,18 @@ class Table:
         results = await Porm.fetch_many(query.get_sql(), cls)
 
         if prefetch:
-            await cls.fetch_related(results, prefetch)
+            results = await cls.fetch_related(results, prefetch)
 
         return results
 
     @classmethod
-    async def update_one(cls: Union[Type[Model], Table], /, item: Model) -> Model:
+    async def update_one(
+        cls: Union[Type[Model], Table],
+        /,
+        item: Model,
+        *,
+        prefetch: FetchType = None,
+    ) -> Model:
 
         query: QueryBuilder = QueryBuilder().update(cls.__tablename__)
 
@@ -297,7 +338,12 @@ class Table:
 
         query = query.where(pk == getattr(item, pk.name))
 
-        return await Porm.fetch_one(with_returning(query), cls)
+        updated = await Porm.fetch_one(with_returning(query), cls)
+
+        if prefetch and updated:
+            [updated] = await cls.fetch_related([updated], prefetch)
+
+        return updated
 
     @classmethod
     async def delete_where(cls: Union[Type[Model], Table], /, criterion: Criterion) -> List[Model]:
@@ -308,19 +354,20 @@ class Table:
         return await Porm.fetch_many(with_returning(query), cls)
 
     @classmethod
-    async def fetch_related(
+    async def _load_relationships_for_items(
         cls: Union[Type[Model], Table],
         /,
         items: List[Union[Model, BaseModel]],
-        _relationships: Tuple[Tuple[_Relationship]],
+        _relationships: FetchType,
     ) -> List[Model]:
-
-        items = [i.copy() for i in items]
+        """Loads relationships, updates items in place"""
         relationship_map = cls._relationship_map()
         type_hints = get_type_hints(cls)
 
-        # TODO: allow for single depth to not have to specify inside tuple
         for relationships in _relationships:
+            if not relationships:
+                continue
+
             relationship = relationships[0]
 
             relationship_table: Type[Table] = None
@@ -356,7 +403,9 @@ class Table:
                 for item in items:
                     foreign_key = getattr(item, relationship.self_field)
 
+                    # Remove unloaded relationship as it has been fetched
                     if foreign_key is None:
+                        setattr(item, relationship_field_name, None)
                         continue
 
                     setattr(item, relationship_field_name, sub_items_map[foreign_key])
@@ -364,6 +413,7 @@ class Table:
             else:
                 items_map = {getattr(i, relationship.self_field): i for i in items}
 
+                # Attach all loaded sub_items to appropriate item
                 for sub_item in sub_items:
                     foreign_key = getattr(sub_item, relationship.other_field)
                     item = items_map[foreign_key]
@@ -373,6 +423,21 @@ class Table:
                     else:
                         getattr(item, relationship_field_name).append(sub_item)
 
-            await cls.fetch_related(sub_items, relationships[1:])
+                # Remove UNLOADED relationships as they have now been loaded
+                for item in items:
+                    if isinstance(getattr(item, relationship_field_name), UNLOADED):
+                        setattr(item, relationship_field_name, [])
 
+            await relationship_table._load_relationships_for_items(sub_items, [relationships[1:]])
+
+    @classmethod
+    async def fetch_related(
+        cls: Union[Type[Model], Table],
+        /,
+        items: List[Union[Model, BaseModel]],
+        _relationships: FetchType,
+    ) -> List[Model]:
+
+        items = [i.copy() for i in items]
+        await cls._load_relationships_for_items(items, _relationships)
         return items
